@@ -1,4 +1,4 @@
-// Copyright (c) 2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2024-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -155,14 +155,16 @@ struct discontinuity_helper
 
 /// Custom warp_exchange class with extra check in scatter_to_striped for out-of-bound accesses.
 template<class T,
-         unsigned int ItemsPerThread,
-         unsigned int WarpSize = ::rocprim::arch::wavefront::min_size()>
+         unsigned int                       ItemsPerThread,
+         unsigned int                       WarpSize = ::rocprim::arch::wavefront::min_size(),
+         ::rocprim::arch::wavefront::target TargetWaveSize
+         = ::rocprim::arch::wavefront::get_target()>
 class custom_warp_exchange
 {
     static_assert(::rocprim::detail::is_power_of_two(WarpSize),
                   "Logical warp size must be a power of two.");
     ROCPRIM_DETAIL_DEVICE_STATIC_ASSERT(
-        WarpSize <= ::rocprim::arch::wavefront::min_size(),
+        WarpSize <= ::rocprim::arch::wavefront::size_from_target<TargetWaveSize>(),
         "Logical warp size cannot be larger than physical warp size.");
 
     static constexpr unsigned int warp_items = WarpSize * ItemsPerThread;
@@ -188,13 +190,13 @@ public:
     /// then writes the values to \p output in a striped manner.
     /// Values in \p ranks that exceed \p WarpSize*ItemsPerThread-1 are not orderer nor written
     /// to output.
-    /// \tparam U - [inferred] the output type.
-    /// \tparam OffsetT - [inferred] the offset type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam OffsetT [inferred] the offset type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [in] ranks - array containing the positions.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [in] ranks array containing the positions.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reuse
     /// Synchronization barrier should be placed before \p storage is reused
@@ -414,10 +416,13 @@ template<typename InputType,
          typename OffsetType,
          typename CountType,
          typename OffsetCountPairType,
-         unsigned int         BlockSize,
-         unsigned int         ItemsPerThread,
-         block_load_method    load_input_method,
-         block_scan_algorithm scan_algorithm>
+         unsigned int                       BlockSize,
+         unsigned int                       ItemsPerThread,
+         block_load_method                  load_input_method,
+         block_scan_algorithm               scan_algorithm,
+         ::rocprim::arch::wavefront::target TargetWaveSize
+         = ::rocprim::arch::wavefront::get_target(),
+         typename Enabled = void>
 class block_helper
 {
 private:
@@ -432,8 +437,8 @@ private:
         = run_length_encode::discontinuity_helper<InputType, equal_op, BlockSize>;
 
     // Warp size.
-    static constexpr unsigned int warp_size
-        = detail::get_min_warp_size(BlockSize, ::rocprim::arch::wavefront::min_size());
+    static constexpr unsigned int warp_size = detail::get_min_warp_size(
+        BlockSize, ::rocprim::arch::wavefront::size_from_target<TargetWaveSize>());
     // Number of warps in block.
     static constexpr unsigned int warps_no = (BlockSize + warp_size - 1) / warp_size;
 
@@ -446,9 +451,9 @@ private:
     // warp_exchange primitives that will be used to perform warp-level scatter_to_striped
     // on offsets and counts.
     using warp_exchange_offsets_type
-        = custom_warp_exchange<OffsetType, ItemsPerThread, /*logical_*/ warp_size>;
+        = custom_warp_exchange<OffsetType, ItemsPerThread, /*logical_*/ warp_size, TargetWaveSize>;
     using warp_exchange_counts_type
-        = custom_warp_exchange<CountType, ItemsPerThread, /*logical_*/ warp_size>;
+        = custom_warp_exchange<CountType, ItemsPerThread, /*logical_*/ warp_size, TargetWaveSize>;
     // Helper class for scattering offsets and counts.
     using warp_scatter_type = run_length_encode::scatter_helper<warp_exchange_offsets_type,
                                                                 warp_exchange_counts_type,
@@ -496,10 +501,8 @@ public:
                                      const unsigned int    block_id,
                                      const std::size_t     grid_size,
                                      const std::size_t     size,
-                                     storage_type_&        storage_)
+                                     storage_type&        storage)
     {
-        storage_type& storage = storage_.get();
-
         static constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
         const std::size_t             block_offset    = block_id * items_per_block;
 
@@ -758,7 +761,63 @@ public:
     }
 };
 
-template<typename Config,
+template<typename InputType,
+         typename OffsetType,
+         typename CountType,
+         typename OffsetCountPairType,
+         unsigned int         BlockSize,
+         unsigned int         ItemsPerThread,
+         block_load_method    load_input_method,
+         block_scan_algorithm scan_algorithm>
+class block_helper<InputType,
+                   OffsetType,
+                   CountType,
+                   OffsetCountPairType,
+                   BlockSize,
+                   ItemsPerThread,
+                   load_input_method,
+                   scan_algorithm,
+                   ::rocprim::arch::wavefront::target::dynamic>
+{
+private:
+    using block_helper_wave32 = block_helper<InputType,
+                                             OffsetType,
+                                             CountType,
+                                             OffsetCountPairType,
+                                             BlockSize,
+                                             ItemsPerThread,
+                                             load_input_method,
+                                             scan_algorithm,
+                                             ::rocprim::arch::wavefront::target::size32>;
+    using block_helper_wave64 = block_helper<InputType,
+                                             OffsetType,
+                                             CountType,
+                                             OffsetCountPairType,
+                                             BlockSize,
+                                             ItemsPerThread,
+                                             load_input_method,
+                                             scan_algorithm,
+                                             ::rocprim::arch::wavefront::target::size64>;
+
+    using dispatch = detail::dispatch_wave_size<block_helper_wave32, block_helper_wave64>;
+
+public:
+    using storage_type = typename dispatch::storage_type;
+
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_WITH_PUSH
+    using storage_type_ = detail::raw_storage<storage_type>;
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_POP
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto process_block(Args&&... args)
+    {
+        return dispatch{}([](auto impl, auto&&... args) { return impl.process_block(args...); },
+                          args...);
+    }
+};
+
+template<typename ArchConfig,
          typename OffsetCountPairType,
          typename InputIterator,
          typename OffsetsOutputIterator,
@@ -778,7 +837,7 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto
     // No need to build the kernel with sleep on a device that does not require it
 }
 
-template<typename Config,
+template<typename ArchConfig,
          typename OffsetCountPairType,
          typename InputIterator,
          typename OffsetsOutputIterator,
@@ -795,7 +854,7 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto
                             const size_t              size)
         -> std::enable_if_t<is_lookback_kernel_runnable<LookbackScanState>()>
 {
-    static constexpr non_trivial_runs_config_params params     = device_params<Config>();
+    static constexpr non_trivial_runs_config_params params     = ArchConfig::params;
     static constexpr unsigned int                   block_size = params.kernel_config.block_size;
     static constexpr unsigned int         items_per_thread  = params.kernel_config.items_per_thread;
     static constexpr block_load_method    load_input_method = params.load_input_method;
@@ -815,7 +874,10 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto
                                          load_input_method,
                                          scan_algorithm>;
 
-    ROCPRIM_SHARED_MEMORY typename block_processor::storage_type_ storage;
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_WITH_PUSH ROCPRIM_SHARED_MEMORY
+    typename detail::raw_storage<typename block_processor::storage_type>
+        storage;
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_POP
 
     const size_t block_id = flat_block_id<block_size, 1, 1>();
 
@@ -834,7 +896,7 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto
                                         block_id,
                                         grid_size,
                                         size,
-                                        storage);
+                                        storage.get());
     }
     else if(valid_in_last_block > 0)
     {
@@ -845,7 +907,7 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto
                                                                     block_id,
                                                                     grid_size,
                                                                     size,
-                                                                    storage);
+                                                                    storage.get());
         // First thread of last block sets the total number of non-trivial runs found and updates
         // the counts with the last run's length if necessary.
         if(threadIdx.x == 0)
